// Keccak hash function interface for libkeccak

unit libkeccak;

interface

procedure whashstringkeccak (Data: string; digest: pointer; hash_size: longint);

procedure mixivkeccak (Data: pointer; datasize: longint);

procedure mixkeykeccak (Data: pointer; datasize: longint);

procedure mixkeykeccakfast (Data: pointer; datasize: longint);

implementation

uses Windows, cmain, Dialogs;

procedure whashstringkeccak (Data: string; digest: pointer; hash_size: longint);
var
  hdll: THandle;
var
  hashnow: procedure (hash_size: longint; Data, digest: pointer; size: longword); stdcall;
begin

  hdll := LoadLibrary(PChar(dll + 'libkeccak.dll'));
  if hdll <> 0 then
    begin
    hashnow := Windows.GetProcAddress(hdll, 'hashnowadv');
    if (@hashnow = nil) then
      begin
      ShowMessage('Functions not found!');
      end;
    end
  else
    ShowMessage('libkeccak not loaded!');
  hashnow(hash_size, PChar(Data), digest, length(Data));
  FreeLibrary(hdll);
end;


procedure XORBuff (I1, I2: Pointer; Size: integer; Dest: Pointer); assembler;
asm
  AND   ECX,ECX
  JZ    @@5
  PUSH  ESI
  PUSH  EDI
  MOV   ESI,EAX
  MOV   EDI,Dest
  @@1:   TEST  ECX,3
  JNZ   @@3
  @@2:   SUB   ECX,4
  JL    @@4
  MOV   EAX,[ESI + ECX]
  XOR   EAX,[EDX + ECX]
  MOV   [EDI + ECX],EAX
  JMP   @@2
  @@3:   DEC   ECX
  MOV   AL,[ESI + ECX]
  XOR   AL,[EDX + ECX]
  MOV   [EDI + ECX],AL
  JMP   @@1
  @@4:   POP   EDI
  POP   ESI
  @@5:
end;


procedure mixivkeccak (Data: pointer; datasize: longint);
var
  hdll: THandle;
  i: integer;
  digest: pointer;
var
  hashnow: procedure (hash_size: longint; Data, digest: pointer; size: longword); stdcall;
begin

  hdll := LoadLibrary(PChar(dll + 'libkeccak.dll'));
  if hdll <> 0 then
    begin
    hashnow := Windows.GetProcAddress(hdll, 'hashnowadv');
    if (@hashnow = nil) then
      begin
      ShowMessage('Functions not found!');
      end;
    end
  else
    ShowMessage('libkeccak not loaded!');


  GetMem(Digest, datasize);


  for i := 0 to 127 do
    begin

    HashNow(datasize * 8, Data, digest, datasize);

    if i mod 2 = 0 then
      XorBuff(Digest, Data, datasize, Data)
    else

      Move(Digest^, Data^, datasize);

    end;

  FreeMem(Digest);

  FreeLibrary(hdll);
end;

procedure mixkeykeccak (Data: pointer; datasize: longint);
var
  hdll: THandle;
  i: integer;
  digest: pointer;
var
  hashnow: procedure (hash_size: longint; Data, digest: pointer; size: longword); stdcall;
begin

  hdll := LoadLibrary(PChar(dll + 'libkeccak.dll'));
  if hdll <> 0 then
    begin
    hashnow := Windows.GetProcAddress(hdll, 'hashnowadv');
    if (@hashnow = nil) then
      begin
      ShowMessage('Functions not found!');
      end;
    end
  else
    ShowMessage('libkeccak not loaded!');


  GetMem(Digest, datasize);


  for i := 0 to 255 do
    begin

    HashNow(datasize * 8, Data, digest, datasize);

    Move(Digest^, Data^, datasize);

    end;

  FreeMem(Digest);

  FreeLibrary(hdll);
end;


procedure mixkeykeccakfast (Data: pointer; datasize: longint);
var
  hdll: THandle;
  i: integer;
  digest: pointer;
var
  hashnow: procedure (hash_size: longint; Data, digest: pointer; size: longword); stdcall;
begin

  hdll := LoadLibrary(PChar(dll + 'libkeccak.dll'));
  if hdll <> 0 then
    begin
    hashnow := Windows.GetProcAddress(hdll, 'hashnowadv');
    if (@hashnow = nil) then
      begin
      ShowMessage('Functions not found!');
      end;
    end
  else
    ShowMessage('libkeccak not loaded!');


  GetMem(Digest, datasize);


  for i := 0 to 1 do
    begin

    HashNow(datasize * 8, Data, digest, datasize);

    Move(Digest^, Data^, datasize);

    end;

  FreeMem(Digest);

  FreeLibrary(hdll);
end;


end.
